@@ -20,8 +20,8 @@ import six
 import grpc
 from grpc._cython import cygrpc
 
-logging.basicConfig()
 _LOGGER = logging.getLogger(__name__)
+_LOGGER.addHandler(logging.NullHandler())
 
 CYGRPC_CONNECTIVITY_STATE_TO_CHANNEL_CONNECTIVITY = {
     cygrpc.ConnectivityState.idle:
