@@ -20,8 +20,8 @@ import grpc
 from grpc import _common
 from grpc._cython import cygrpc
 
-logging.basicConfig()
 _LOGGER = logging.getLogger(__name__)
+_LOGGER.addHandler(logging.NullHandler())
 
 
 class _AuthMetadataContext(
