@@ -21,8 +21,8 @@ import logging
 
 import six
 
-logging.basicConfig()
 _LOGGER = logging.getLogger(__name__)
+_LOGGER.addHandler(logging.NullHandler())
 
 
 class Outcome(six.with_metaclass(abc.ABCMeta)):
