@@ -17,8 +17,8 @@ import logging
 
 from concurrent import futures
 
-logging.basicConfig()
 _LOGGER = logging.getLogger(__name__)
+_LOGGER.addHandler(logging.NullHandler())
 
 
 def _wrap(behavior):
