@@ -15,6 +15,7 @@
 
 from concurrent import futures
 import unittest
+import logging
 
 import grpc
 
@@ -46,4 +47,5 @@ class ChannelArgsTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
+    logging.basicConfig()
     unittest.main(verbosity=2)
