@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import unittest
+import logging
 
 import grpc
 
@@ -118,4 +119,5 @@ class EmptyMessageTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
+    logging.basicConfig()
     unittest.main(verbosity=2)
