@@ -15,6 +15,7 @@
 
 import threading
 import unittest
+import logging
 
 import grpc
 
@@ -656,4 +657,5 @@ class MetadataCodeDetailsTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
+    logging.basicConfig()
     unittest.main(verbosity=2)
