@@ -15,6 +15,7 @@
 
 import threading
 import unittest
+import logging
 
 import grpc
 from grpc import _channel
@@ -253,4 +254,5 @@ class ResourceExhaustedTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
+    logging.basicConfig()
     unittest.main(verbosity=2)
