@@ -1,33 +1,18 @@
 #!/usr/bin/env ruby
 
-# Copyright 2015, Google Inc.
-# All rights reserved.
+# Copyright 2015 gRPC authors.
 #
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are
-# met:
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-#     * Redistributions of source code must retain the above copyright
-# notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above
-# copyright notice, this list of conditions and the following disclaimer
-# in the documentation and/or other materials provided with the
-# distribution.
-#     * Neither the name of Google Inc. nor the names of its
-# contributors may be used to endorse or promote products derived from
-# this software without specific prior written permission.
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 # client is a testing tool that accesses a gRPC interop testing server and runs
 # a test on it.
@@ -110,7 +95,7 @@ end
 
 # creates a test stub that accesses host:port securely.
 def create_stub(opts)
-  address = "#{opts.host}:#{opts.port}"
+  address = "#{opts.server_host}:#{opts.server_port}"
 
   # Provide channel args that request compression by default
   # for compression interop tests
@@ -127,7 +112,7 @@ def create_stub(opts)
     creds = ssl_creds(opts.use_test_ca)
     stub_opts = {
       channel_args: {
-        GRPC::Core::Channel::SSL_TARGET => opts.host_override
+        GRPC::Core::Channel::SSL_TARGET => opts.server_host_override
       }
     }
 
@@ -696,13 +681,13 @@ class NamedTests
   # Send probing message for compressed request on the server, to see
   # if it's implemented.
   def send_probe_for_compressed_request_support(&send_probe)
-    bad_status_occured = false
+    bad_status_occurred = false
 
     begin
       send_probe.call
     rescue GRPC::BadStatus => e
       if e.code == GRPC::Core::StatusCodes::INVALID_ARGUMENT
-        bad_status_occured = true
+        bad_status_occurred = true
       else
         fail AssertionError, "Bad status received but code is #{e.code}"
       end
@@ -711,26 +696,26 @@ class NamedTests
     end
 
     assert('CompressedRequest probe failed') do
-      bad_status_occured
+      bad_status_occurred
     end
   end
 
 end
 
 # Args is used to hold the command line info.
-Args = Struct.new(:default_service_account, :host, :host_override,
-                  :oauth_scope, :port, :secure, :test_case,
+Args = Struct.new(:default_service_account, :server_host, :server_host_override,
+                  :oauth_scope, :server_port, :secure, :test_case,
                   :use_test_ca)
 
-# validates the the command line options, returning them as a Hash.
+# validates the command line options, returning them as a Hash.
 def parse_args
   args = Args.new
-  args.host_override = 'foo.test.google.fr'
+  args.server_host_override = 'foo.test.google.fr'
   OptionParser.new do |opts|
     opts.on('--oauth_scope scope',
             'Scope for OAuth tokens') { |v| args['oauth_scope'] = v }
     opts.on('--server_host SERVER_HOST', 'server hostname') do |v|
-      args['host'] = v
+      args['server_host'] = v
     end
     opts.on('--default_service_account email_address',
             'email address of the default service account') do |v|
@@ -738,9 +723,11 @@ def parse_args
     end
     opts.on('--server_host_override HOST_OVERRIDE',
             'override host via a HTTP header') do |v|
-      args['host_override'] = v
+      args['server_host_override'] = v
+    end
+    opts.on('--server_port SERVER_PORT', 'server port') do |v|
+      args['server_port'] = v
     end
-    opts.on('--server_port SERVER_PORT', 'server port') { |v| args['port'] = v }
     # instance_methods(false) gives only the methods defined in that class
     test_cases = NamedTests.instance_methods(false).map(&:to_s)
     test_case_list = test_cases.join(',')
@@ -749,7 +736,7 @@ def parse_args
     opts.on('--use_tls USE_TLS', ['false', 'true'],
             'require a secure connection?') do |v|
       args['secure'] = v == 'true'
-p    end
+    end
     opts.on('--use_test_ca USE_TEST_CA', ['false', 'true'],
             'if secure, use the test certificate?') do |v|
       args['use_test_ca'] = v == 'true'
@@ -759,7 +746,7 @@ p    end
 end
 
 def _check_args(args)
-  %w(host port test_case).each do |a|
+  %w(server_host server_port test_case).each do |a|
     if args[a].nil?
       fail(OptionParser::MissingArgument, "please specify --#{a}")
     end
